package ucot.core;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import ucot.heuristic.AbbottsHeuristic;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.InputInterface;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.output.Output;
import ucot.output.OutputInterface;
import ucot.parser.ParsedData;
import ucot.parser.ParserInterface;
import ucot.parser.SimpleInputParser;
import ucot.ui.UIInterface;
import ucot.ui.gui.TestUI;

public class Core implements ControlInterface{

	InputCollection inputs = new InputCollection();
	UseCaseCollection usecases = new UseCaseCollection();
	Vector<AnalyzeModel> models = new Vector<AnalyzeModel>();
	Vector<HeuristicInterface> heuristics = new Vector<HeuristicInterface>();
	Vector<ParserInterface> parsers = new Vector<ParserInterface>();
	Vector<OutputInterface> outputs = new Vector<OutputInterface>();
	UIInterface ui = new TestUI();
	ParserInterface currentParser = null;
	HeuristicInterface currentHeuristic = null;
	

	public Core(){
		System.out.println("Launching UCOT-core");
		models.add(new AnalyzeModel());
		currentParser = new SimpleInputParser();
		currentHeuristic = new AbbottsHeuristic();
		outputs.add(new Output());
		heuristics.add(currentHeuristic);
		parsers.add(currentParser);
	}
	
	public void shutdown() {			
	}

    /**
     * Loads usecases from source pointed out by url into loacl variable usecases.
     */
	public void loadUseCases(URL url) {
		if (url == null)
			return;
		
        InputInterface input = inputs.getInputForUrl(url);
        
        if (input == null)
        	return;
        
        try {            
        		this.usecases.merge(input.read(url));
        } catch (IOException e) {
                System.err.println(e.getMessage());
        }       
        
        if (ui != null)
        	ui.useCasesLoaded();
    }

	public void loadAnalyzeModel(URL url) {
		if (ui != null)
        	ui.analyzeModelLoaded();
	}

	public UseCaseCollection getUseCaseCollection() {		
		return usecases;
	}

	public void addToAnalyzeModel(UseCase usecase, AnalyzeModel model) throws Exception {
		if (currentHeuristic == null || currentParser == null)
			return;
		try {
			ParsedData data = currentParser.parse(usecase);
			currentHeuristic.doHeuristic(data,model);
		} catch(Exception e) {
			System.err.println(e.getMessage());
		}
		
		if (ui != null)
        	ui.useCaseAdded();
	}

	public Vector<AnalyzeModel> getAnalyzeModels() {		
		return models;
	}

	public Vector<ParserInterface> getParsers() {		
		return parsers;
	}

	public Vector<HeuristicInterface> getHeuristics() {
		return heuristics;
	}

	public Vector<OutputInterface> getOutputs() { 
		return outputs;
	}

	public InputCollection getInputs() {
		return inputs;
	}

	public void setCurrentParser(ParserInterface parser) {
		currentParser = parser;
		
	}

	public void setCurrentHeuristic(HeuristicInterface heuristic) {
		currentHeuristic = heuristic;
		
	}

	public ParserInterface getCurrentParser() {
		return currentParser;
	}

	public HeuristicInterface getCurrentheuristic() {
		return currentHeuristic;
	}

	public void output(URL url, OutputInterface output, AnalyzeModel model) throws Exception {
		if (output != null)
			output.output(model,url);
		if (ui != null)
			ui.exportDone();
	}

	public void saveAnalyzeModel(URL url) {
		
	}

}
