package ucot.parser;

import java.util.Properties;
import java.util.StringTokenizer;

import ucot.input.UseCase;

public class SimpleInputParser implements ParserInterface {
	private Properties options = new Properties();
	
	public ParsedData parse(UseCase usecase) throws Exception {
		if ( usecase == null ) return null;
		
		ParsedData pd = new ParsedData();
		pd.setUseCase(usecase);
		StringTokenizer st = null;
		Sentence sentence = null;
		Word word = null,subj = null,pred = null,obj = null;
		int count = 0;
		String wordClass = "noun";
		//Go through all use case steps
		for(int i=0;i<usecase.getStepCount();i++) {
			//Separate each word from the use case line
			st = new StringTokenizer(usecase.getStep(i),",");
			sentence = new Sentence();
			
			count=0;
			
			//And go through them all
			while(st.hasMoreTokens()) {
				//there should only be 3 words
				if (count > 2) break;
				//Usually the words are noun
				wordClass = "noun";				
				//allthough the second word is always a verb
				if (count == 1) wordClass = "verb";
				
				//Create a word objcet from the text
				word = new Word(st.nextToken().trim(),wordClass);
				
				//And add it to our sentence
				sentence.addWord(word);				
				
				count++;
			}
			
			//In our simple input:			
			//First word is always subject			
			subj = sentence.getWords().get(0);
			//Second word is always predicate
			pred = sentence.getWords().get(1);
			//Third word is always object
			obj = sentence.getWords().get(2);

			//Incase we didn't get 3 words, there should be no link and sentence shouldn't be added(?)
			if (null != subj && null != pred && null != obj){
				pred.addLink(new Link("subject",subj));
				pred.addLink(new Link("object",obj));
				
				pd.addSentence(sentence);
			}
		}
		
		return pd;
	}

	public String getName() {		
		return "Simple input parser";
	}

	public void setOptions(Properties options) {
		this.options = options;		
	}

	public Properties getOptions() {
		return options;
	}
	
}
